{
  Copyright 2023-2023 Michalis Kamburelis.

  This file is part of "Castle Game Engine".

  "Castle Game Engine" is free software; see the file COPYING.txt,
  included in this distribution, for details about the copyright.

  "Castle Game Engine" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

  ----------------------------------------------------------------------------
}

{ Main view, where most of the application logic takes place. }
unit GameViewMain;

interface

uses Classes,
  CastleVectors, CastleComponentSerialize, CastleTransform,
  CastleUIControls, CastleControls, CastleKeysMouse, CastleScene;
  // GameSceneSubAnimations;

type
  { Main view, where most of the application logic takes place. }
  TViewMain = class(TCastleView)
  published
    { Components designed using CGE editor.
      These fields will be automatically initialized at Start. }
    LabelFps: TCastleLabel;
    ModelParent: TCastleTransform;
    CheckboxLoop: TCastleCheckbox;
  private
    {Only for subanimations that exist; crashes otherwise }
    // ModelScene: TSceneSubAnimations;
    // ModelScene2: TSceneSubAnimations;
    ModelScene: TCastleScene;
    ModelScene2: TCastleScene;
    procedure LoadModel(const ModelUrl, ModelUrl2: String);
    // procedure ClickPlayAnimation(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Start; override;
    procedure Update(const SecondsPassed: Single; var HandleInput: Boolean); override;
  end;

var
  ViewMain: TViewMain;

implementation

uses SysUtils;

{ TViewMain ----------------------------------------------------------------- }

constructor TViewMain.Create(AOwner: TComponent);
begin
  inherited;
  DesignUrl := 'castle-data:/gameviewmain.castle-user-interface';
end;

procedure TViewMain.Start;
begin
  inherited;

  // load some initial model
  LoadModel('castle-data:/tremulous-data/sandy/lower.md3', 'castle-data:/tremulous-data/sandy/lower.md3');
end;

procedure TViewMain.LoadModel(const ModelUrl, ModelUrl2: String);
var
  NewButton: TCastleButton;
  SubAnimName: String;
begin

  { for when the subanimations exist only }
  // ModelScene := TSceneSubAnimations.Create(FreeAtStop);
  // ModelScene2 := TSceneSubAnimations.Create(FreeAtStop);
  ModelScene.Load(ModelUrl);
  ModelScene2.Load(ModelUrl2);
  { Do not call PlayAnimation, as then both sensors started by PlayAnimation,
    and ForceAnimationPose done by PlaySubAnimation, would try to update the model
    -- it would be undefined which animation is eventually visible. }
  // ModelScene.PlayAnimation('animation', true);
  ModelParent.Add(ModelScene);
  ModelParent.Add(ModelScene2);
  { This is only if the model has animations - the current test does not. }
  // ModelScene.PlaySubAnimation('NSPA_IDLE', Checkboxloop.Checked);
  // ModelScene2.PlaySubAnimation('NSPA_IDLE', Checkboxloop.Checked);
end;

{ procedure TViewMain.ClickPlayAnimation(Sender: TObject);
var
  SubAnimationName: String;
  SubAnimationName2: String;
begin
  SubAnimationName := 'NSPA_RUN';
  SubAnimationName2 := 'NSPA_JUMPB';
  //(Sender as TCastleButton).Caption: remember to replace this back after if the above change fails
  ModelScene.PlaySubAnimation(SubAnimationName, CheckboxLoop.Checked);
  ModelScene2.PlaySubAnimation(SubAnimationName2, CheckboxLoop.Checked);
end; }

procedure TViewMain.Update(const SecondsPassed: Single; var HandleInput: Boolean);
begin
  inherited;
  { This virtual method is executed every frame (many times per second). }
  Assert(LabelFps <> nil, 'If you remove LabelFps from the design, remember to remove also the assignment "LabelFps.Caption := ..." from code');
  LabelFps.Caption := 'FPS: ' + Container.Fps.ToString;
end;

end.
